"""This module is part of the PygLibsX library.
It is licensed under an open-source, make-shift license by Matthew Roe.
That will change when it is more complete and I know what I want ot do with it."""

import pygame
from OpenGL.GL import *
import os
import texture
import math3d

def MTL(filename, path='', max_size="default"):
    contents = {}
    mtl = None
    for line in open(os.path.join(path,filename), "r"):
        if line.startswith('#'): continue
        values = line.split()
        if not values: continue
        if values[0] == 'newmtl':
            mtl = contents[values[1]] = {}
        elif mtl is None:
            raise ValueError, "mtl file doesn't start with newmtl stmt"
        elif values[0] == 'map_Kd':
            # load the texture referred to by this declaration
            mtl[values[0]] = values[1]
            tex = texture.image(os.path.join(path,mtl['map_Kd']))
            texid = tex.image
            mtl['tex'] = texid
        elif values[0]=="Kd":
            mtl[values[0]] = map(float, values[1:])
        else:
            pass
    return contents
 
class OBJ:
    def __init__(self, filename, swapyz=False, max_size="default",
                 collision = "Cube"):
        """Loads a Wavefront OBJ file. """
        self.vertices = []
        self.normals = []
        self.texcoords = []
        self.faces = []
 
        material = None
        for line in open(filename, "r"):
            if line.startswith('#'): continue
            values = line.split()
            if not values: continue
            if values[0] == 'v':
                v = map(float, values[1:4])
                if swapyz:
                    v = v[0], v[2], v[1]
                self.vertices.append(v)
            elif values[0] == 'vn':
                v = map(float, values[1:4])
                if swapyz:
                    v = v[0], v[2], v[1]
                self.normals.append(v)
            elif values[0] == 'vt':
                self.texcoords.append(map(float, values[1:3]))
            elif values[0] in ('usemtl', 'usemat'):
                material = values[1]
            elif values[0] == 'mtllib':
                self.mtl = MTL(values[1], os.path.split(filename)[0], max_size)
            elif values[0] == 'f':
                face = []
                texcoords = []
                norms = []
                for v in values[1:]:
                    w = v.split('/')
                    face.append(int(w[0]))
                    if len(w) >= 2 and len(w[1]) > 0:
                        texcoords.append(int(w[1]))
                    else:
                        texcoords.append(0)
                    if len(w) >= 3 and len(w[2]) > 0:
                        norms.append(int(w[2]))
                    else:
                        norms.append(0)
                self.faces.append((face, norms, texcoords, material))
 
        self.gl_list = glGenLists(1)
        glNewList(self.gl_list, GL_COMPILE)
        glEnable(GL_TEXTURE_2D)
        glFrontFace(GL_CCW)
        for face in self.faces:
            vertices, normals, texture_coords, material = face
 
            mtl = self.mtl[material]
            if 'map_Kd' in mtl:
                # use diffuse texmap
                glBindTexture(GL_TEXTURE_2D, mtl['tex'])
            else:
                # just use diffuse colour
                glColor(*mtl['Kd'])
 
            glBegin(GL_POLYGON)
            for i in range(0, len(vertices)):
                if normals[i] > 0:
                    glNormal3fv(self.normals[normals[i] - 1])
                if texture_coords[i] > 0:
                    glTexCoord2fv(self.texcoords[texture_coords[i] - 1])
                glVertex3fv(self.vertices[vertices[i] - 1])
            glEnd()
        glDisable(GL_TEXTURE_2D)
        glEndList()

        self.collision_geom_type = collision
        self.make_cgeom()

    def make_cgeom(self):
        x = y = z = []
        for v in self.vertices:
            x.append(abs(v[0]))
            y.append(abs(v[1]))
            z.append(abs(v[2]))

        x = max(x)
        y = max(y)
        z = max(z)

        if self.collision_geom_type == "Cube":
            self.collision_geom = math3d.Cube((0, 0, 0), x, y, z)
        if self.collision_geom_type == "Sphere":
            self.collision_geom = math3d.Sphere((0, 0, 0), max ([x, y, z]))
        if self.collision_geom_type == "Vertex":
            self.collision_geom = math3d.Vertex((0, 0, 0))
        return self.collision_geom

    def render(self, pos, rotation, color=(1.0,1.0,1.0), alpha=1.0):
        glPushMatrix()
        glColor3f(*color)
        glTranslatef(*pos)
        glRotatef(rotation[0], 1, 0, 0)
        glRotatef(rotation[1], 0, 1, 0)
        glRotatef(rotation[2], 0, 0, 1)
        glCallList(self.gl_list)
        glPopMatrix()
        return None

